package com.topcoder.db.generation.xsd2ddl.parsers;

import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;

import org.w3c.dom.Node;

import com.topcoder.db.generation.xsd2ddl.DefinitionParser;
import com.topcoder.db.generation.xsd2ddl.ParseException;
import com.topcoder.db.generation.xsd2ddl.data.Database;
import com.topcoder.db.generation.xsd2ddl.data.Field;
import com.topcoder.db.generation.xsd2ddl.data.FieldType;
import com.topcoder.db.generation.xsd2ddl.data.Key;
import com.topcoder.db.generation.xsd2ddl.data.Table;
import com.topcoder.db.generation.xsd2ddl.data.TableType;

/**
 * This is the default implementation of the DefinitionParser interface.
 * The source of the definition is an InputStream. The content of the stream is XSD
 * (XML schema) content (a restricted set).
 * 
 * The parsing process produces a Database data class that contains instances of all the
 * other classes from the data package (Field, FieldType, Table, TableType, Key).
 * 
 * Essentially what this interface does is converting the database definition from an
 * XSD format into a generic, implementation independent format represented by the classes in the
 * data package.
 * 
 * There are several conversion conventions. FieldTypes are represented by XSD simpleTypes.
 * TableTypes are represented by XSD complexTypes. Fields are represented by XSD elements
 * and attributes (nested in complexTypes). Tables are represented by direct XSD elements.
 * Primary keys and foreign keys are represented by XSD keys and keyrefs.
 * The sample XSD should be consulted for full details. 
 * 
 * Note:
 * 
 * This implementations has a public constructor taking a configuration
 * namespace (String) as parameter but it doesn't really read any config info.
 * It is defined just to fulfill the interface constract.
 * 
 * Thread safety:
 * 
 * The implementation keeps caches of the field and table types and primary key
 * definitions during parsing. This makes the parse call not thread safe.
 * This can be easily changed by passing all maps as arguments instead of using
 * attributes but it would make the method signatures ugly. Since thread safety
 * isn't a requirement and because it can be achieved easily and efficiently by using 
 * separate parser and DefinitionUtil classes, it doesn't make sense to do it. 
 * 
 * @author TCSDESIGNER
 */
public class XSDParser implements DefinitionParser {
	
	/**
	 * Field type cache.
	 * It is a mapping from field type names (String) to field types (FieldType).
	 * Cannot be null, contain nulls, empty or all spaces strings as keys or values.
	 * Used in the parse methods.
	 */
	private Map fieldTypes = new HashMap();
	
	/**
	 * Table type cache.
	 * It is a mapping from table type names (String) to table types (TableType).
	 * Cannot be null, contain nulls, empty or all spaces strings as keys or values.
	 * Used in the parse methods.
	 */
	private Map tableTypes = new HashMap();
	
	/**
	 * Primary key cache.
	 * It is a mapping from primary key names (String) to the tables owning them (Table)
	 * and indirectly to the Key itself (Table.getPrimaryKey()).
	 * Cannot be null, contain nulls, empty or all spaces strings as keys or values.
	 * Used in the parse methods.
	 */
	private Map primaryKeysToTables = new HashMap();
	
	/**
	 * Constructor. Does nothing.
	 * 
	 * @param namespace the configuration namespace
	 * 
	 * @throws IllegalArgumentException if the argument is null, empty or all spaces
	 */
	public XSDParser(String namespace) throws IllegalArgumentException {
	}

	/**
	 * Default constructor. Does nothing.
	 */
	public XSDParser() {
	}
	
	/**
	 * Parses the database definition from the input stream and creates a Database
	 * structure (including Field, FileType, Key, Table, TableType instances).
	 * 
	 * The format of the content of the input stream is a restricted set of an XSD.
	 * 
	 * Implementation details:
	 * 
	 * There are two parts here, one is the actual parsing and one is ordering the tables
	 * in such a way that a table having foreign keys doesn't appear before all the
	 * referred tables.
	 * 
	 * - creates a DOM representation of the XML from the input stream
	 * - iterate elements belonging to root
	 * - if element is XSD simpleType, call parseFieldType and cache result in fieldTypes
	 * - if element is XSD complexType, call parseTableType and cache result in tableTypes
	 * - if element is XSD element, call parseTable and add result to a list
	 * - end iteration
	 * 
	 * - for i = 0 to list.length - 1
	 * - iterate list from i to end and select any table that refers in its foreign keys
	 *   at most tables from 0 to i-1
	 * - if no table is found it means circular references exist (error condition)
	 * - swap table at position i with the found table 
	 * - next i 
	 * 
	 * - creates a Database and adds to it all tables from the list
	 * 
	 * @param definitionStream the XSD definition stream to read from
	 * 
	 * @return the database definition object
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 * @throws IOException if an I/O error occurs while reading from the stream
	 * @throws ParseException wraps a potential XML exception or can indicate a missing required entity
	 */
	public Database parse(InputStream definitionStream) throws IllegalArgumentException, IOException, ParseException {
		return null;
	}
	
	/**
	 * Parses a field type represented by an XSD simpleType.
	 * 
	 * Implementation details:
	 * 
	 * - create a FieldType
	 * - read the field type name from the name attribute
	 * - read the field data type from restriction / base
	 * - iterate all restrictions and fill the FieldTypes accordingly
	 * - return FieldType
	 * 
	 * @param node the node containing the field type
	 * 
	 * @return the parsed field type
	 * @throws IllegalArgumentException if the argument is null
	 * @throws IOException if an I/O error occurs while reading from the stream
	 * @throws ParseException wraps a potential XML exception or can indicate a missing required entity
	 */
	protected FieldType parseFieldType(Node node) throws IllegalArgumentException, IOException, ParseException { // xsd:simpleType
		return null;
	}
	
	/**
	 * Parses a field represented by an XSD element or attribute node.
	 * 
	 * Implementation details:
	 * 
	 * - create a Field
	 * - read the field name from the name attribute
	 * - read the nullable indicator from the nillable attribute
	 * - if type attribute is present get field type from the field type cache
	 * - else must have a nested simpleType so call parseFieldType to parse it
	 * - return Field
	 * 
	 * @param node the node containing the field
	 * 
	 * @return the parsed field
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 * @throws IOException if an I/O error occurs while reading from the stream
	 * @throws ParseException wraps a potential XML exception or can indicate a missing required entity
	 */
	protected Field parseField(Node node) throws IllegalArgumentException, IOException, ParseException { // xsd:complexType / xsd:element/attribute
		return null;
	}
	
	/**
	 * Parses a table type represented by an XSD complexType.
	 * 
	 * Implementation details:
	 * 
	 * - create a TableType
	 * - read the table type name from the name attribute
	 * - iterate all nodes from sequence subnode
	 * - use parseField to parse each node and add parse field to TableType
	 * - return TableType
	 * 
	 * @param node the node containing the table type
	 * 
	 * @return the parsed table type
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 * @throws IOException if an I/O error occurs while reading from the stream
	 * @throws ParseException wraps a potential XML exception or can indicate a missing required entity
	 */
	protected TableType parseTableType(Node node) throws IllegalArgumentException, IOException, ParseException { // xsd:complexType
		return null;
	}

	/**
	 * Parses a table represented by an XSD element node.
	 * 
	 * Implementation details:
	 * 
	 * - if key subnode exists get its name and see if there is a dummy table in the primaryKeysToTables cache
	 *   (this table is created in advance if a foreign key definition appears before the referred table)
	 * - else create a new Table
	 * - read the table name from the name attribute
	 * - if type attribute is present get table type from the table type cache
	 * - else must have a nested complexType so call parseTableType to parse it
	 * - for each keyref subnode call parseForeignKey
	 * - if key subnode exists call parsePrimaryKey
	 * - return Table
	 * 
	 * @param node the node containing the field
	 * 
	 * @return the parsed field
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 * @throws IOException if an I/O error occurs while reading from the stream
	 * @throws ParseException wraps a potential XML exception or can indicate a missing required entity
	 */
	protected Table parseTable(Node node) throws IllegalArgumentException, IOException, ParseException { // xsd:element
		return null;
	}
	
	/**
	 * Parses a primary key represented by an XSD key node.
	 * 
	 * Implementation details:
	 * 
	 * - read the key name from the name attribute
	 * - if the key name exists in the primaryKeysToTables cache, get the dummy table from there and
	 *   the dummy primary key from the dummy table (it is a dummy key with no fields added by a foreign key reference)
	 * - else create key and put it in cache
	 * - for each field subnode add the xpath attribute as key field (@ is stripped if present)
	 * - return Key
	 * 
	 * @param node the node containing the key
	 * @param table the owner table (needed to be able to complete the cache)
	 * 
	 * @return the parsed key
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 * @throws IOException if an I/O error occurs while reading from the stream
	 * @throws ParseException wraps a potential XML exception or can indicate a missing required entity
	 */
	protected Key parsePrimaryKey(Node node, Table table) throws IllegalArgumentException, IOException, ParseException { // xsd:key
		return null;
	}

	/**
	 * Parses a foreign key represented by an XSD key node.
	 * 
	 * Implementation details:
	 * 
	 * - create a Key
	 * - read the key name from the name attribute
	 * - for each field subnode add the xpath attribute as key field (@ is stripped if present)
	 * - read refer attribute
	 * - if the primaryKeysToTables contains refer value then get the Table from the cache and the 
	 *   primary Key from the Table and complete the referred foreign Key fields
	 * - if the cache does not have an entry for refer value then create a dummy Table with a
	 *   dummy primary key and put it in the cache and complete the referred foreign Key fields
	 *   (this dummy objects will be filled later by parsePrimaryKey when the primary key definition occurs)
	 * - return Key
	 * 
	 * @param node the node containing the key
	 * 
	 * @return the parsed key
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 * @throws IOException if an I/O error occurs while reading from the stream
	 * @throws ParseException wraps a potential XML exception or can indicate a missing required entity
	 */
	protected Key parseForeignKey(Node node) throws IllegalArgumentException, IOException, ParseException { // xsd:keyref
		return null;
	}

}
