package com.topcoder.db.generation.xsd2ddl.renderers;

import java.io.IOException;
import java.io.OutputStream;

import com.topcoder.db.generation.xsd2ddl.DefinitionRenderer;
import com.topcoder.db.generation.xsd2ddl.RenderException;
import com.topcoder.db.generation.xsd2ddl.TypeMapping;
import com.topcoder.db.generation.xsd2ddl.data.Database;
import com.topcoder.db.generation.xsd2ddl.data.Table;
import com.topcoder.util.config.ConfigManagerException;

/**
 * This class in an implementation of the DefinitionRenderer interface that produces
 * DDL (SQL CREATE and ALTER statements) for Oracle 10g.
 *
 * The rendering process takes as input a Database object that contains instances of all the
 * other classes from the data package (Field, FieldType, Table, TableType, Key).
 *
 * The destination of the rendering in an output stream. The content that is written to the output
 * is Oracle 10g specific DDL. The output takes into account field types and field constraints.
 * For the purpose of the type conversion it uses a ConfigMapping instance. This instance
 * uses the mapping defined in a config file (see the sample config file). This allows flexibility
 * in the chosen the conversion types. It can also use a custom TypeMapping implementation if
 * the mapping follows a more complex algorithm that the configuration rules can express. 
 * 
 * Note:
 * 
 * This class has a constructor taking a namespace. The namespace is used to load the type mapping
 * class to use (the namespace is also passed to the type mapping constructor).
 * There is also a constructor that allows passing an already built TypeMapping.
 * 
 * Thread safety:
 * 
 * This class is thread safe because it is imutable.
 * 
 * @author TCSDESIGNER
 */
public class OracleCreateRenderer implements DefinitionRenderer {
	
	/**
	 * The type mapping instance to be used for type conversions (between XSD types and Oracle 10g types).
	 * Cannot be null.
	 * Set in the constructor, used in render.
	 */
	private TypeMapping typeMapping = null;
	
	/**
	 * Constructor.
	 * 
	 * Implementation details:
	 * 
	 * - reads type_mapping_class from the namespace
	 * - invokes through reflection the constructor that takes a namespace as argument
	 * - sets typeMapping to the obtained instance
	 * 
	 * @param namespace the configuration namespace
	 * 
	 * @throws IllegalArgumentException if the argument is null, empty or all space
	 * @throws ConfigManagerException if any configuration exception occurs or a required field is missing
	 */
	public OracleCreateRenderer(String namespace) throws IllegalArgumentException, ConfigManagerException {
	}

	/**
	 * Constructor. Sets typeMapping to the given value
	 * 
	 * @param typeMapping the type mapping instance
	 * 
	 * @throws IllegalArgumentException if the argument is null
	 */
	public OracleCreateRenderer(TypeMapping typeMapping) throws IllegalArgumentException {
	}

	/**
	 * Renders the database definition from input Database object (can include
	 * Field, FileType, Key, Table, TableType instances) to a stream.
	 * 
	 * The output is a Oracle 10g compatible DDL script.
	 * 
	 * Implementation details:
	 * 
	 * - for each table in database 
	 * - output "CREATE TABLE " + table.getTableName() + " (" + "\r\n"
	 * 
	 * - for each field of the table
	 * - output field.getFieldName() + " " + typeMapping.convertType(field.getFieldType())
	 * - if field.isNullable() == false output " NOT NULL"
	 * - if not last field output "," + "\r\n"
	 * - end for each field 
	 * - output ");" + "\r\n"
	 * 
	 * - if table has primary key
	 * - output "ALTER TABLE " + table.getTableName() + " ADD PRIMARY KEY " + key.getKeyName() + " ("
	 * - for each key field, output field.getFieldName() + "," ("," not printed for the last one)
	 * - output ");" + "\r\n"
	 * - end if
	 * 
	 * - for each foreign key
	 * - output "ALTER TABLE " + table.getTableName() + " ADD FOREIGN KEY " + ket.getKeyName() + " ("
	 * - for each key field, output field.getFieldName() + "," ("," not printed for the last one)
	 * - output ")\r\n"
	 * - output "REFERENCES " + key.getReferredTable().getTableName() + " ("
	 * - for each key field in the referred key, output field.getFieldName() + "," ("," not printed for the last one)
	 * - output ");\r\n"
	 * - end for each foreign key
	 * 
	 * - for each field of the table
	 * - counter = 1
	 * - iterate all field type constraints in turn (those with value different than -1 or null)
	 * - output "ALTER TABLE " + table.getTableName() + " ADD CONSTRAINT " + tableName + "_" + fieldName + "_" + (counter++) + " CHECK ("
	 * - for length related constraints use comparison between LENGTH(fieldName) and length/minLength/maxLength
	 * - for min/max related contraints use comparison between fieldName and min/max Inclusive/Exclusive
	 * - for pattern constraint use REGEXP_LIKE(filedName, 'pattern')
	 * - for enumeration constraint use fieldName IN (comma_separate_enumeration_values) 
	 * - output ");\r\n" 
	 * - end for each field
	 * 
	 * - end for each table
	 * 
	 * @param definition the database definition
	 * @param renderStream the destination stream
	 * 
	 * @throws IllegalArgumentException if any argument is null
	 * @throws IOException if an I/O error occurs while working with the stream
	 * @throws RenderException if a renderer implementation specific error occurs
	 */
	public void render(Database definition, OutputStream renderStream) throws IllegalArgumentException, IOException, RenderException {
	}

}
